# Generated from PartiQLParser.g4 by ANTLR 4.13.0
from antlr4 import *
if "." in __name__:
    from .PartiQLParser import PartiQLParser
else:
    from PartiQLParser import PartiQLParser

# This class defines a complete generic visitor for a parse tree produced by PartiQLParser.

class PartiQLParserVisitor(ParseTreeVisitor):

    # Visit a parse tree produced by PartiQLParser#root.
    def visitRoot(self, ctx:PartiQLParser.RootContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#QueryDql.
    def visitQueryDql(self, ctx:PartiQLParser.QueryDqlContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#QueryDml.
    def visitQueryDml(self, ctx:PartiQLParser.QueryDmlContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#QueryDdl.
    def visitQueryDdl(self, ctx:PartiQLParser.QueryDdlContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#QueryExec.
    def visitQueryExec(self, ctx:PartiQLParser.QueryExecContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#explainOption.
    def visitExplainOption(self, ctx:PartiQLParser.ExplainOptionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#asIdent.
    def visitAsIdent(self, ctx:PartiQLParser.AsIdentContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#atIdent.
    def visitAtIdent(self, ctx:PartiQLParser.AtIdentContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#byIdent.
    def visitByIdent(self, ctx:PartiQLParser.ByIdentContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#symbolPrimitive.
    def visitSymbolPrimitive(self, ctx:PartiQLParser.SymbolPrimitiveContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#dql.
    def visitDql(self, ctx:PartiQLParser.DqlContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#execCommand.
    def visitExecCommand(self, ctx:PartiQLParser.ExecCommandContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#qualifiedName.
    def visitQualifiedName(self, ctx:PartiQLParser.QualifiedNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#tableName.
    def visitTableName(self, ctx:PartiQLParser.TableNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#tableConstraintName.
    def visitTableConstraintName(self, ctx:PartiQLParser.TableConstraintNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#columnName.
    def visitColumnName(self, ctx:PartiQLParser.ColumnNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#columnConstraintName.
    def visitColumnConstraintName(self, ctx:PartiQLParser.ColumnConstraintNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ddl.
    def visitDdl(self, ctx:PartiQLParser.DdlContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#CreateTable.
    def visitCreateTable(self, ctx:PartiQLParser.CreateTableContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#CreateIndex.
    def visitCreateIndex(self, ctx:PartiQLParser.CreateIndexContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#DropTable.
    def visitDropTable(self, ctx:PartiQLParser.DropTableContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#DropIndex.
    def visitDropIndex(self, ctx:PartiQLParser.DropIndexContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#tableDef.
    def visitTableDef(self, ctx:PartiQLParser.TableDefContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ColumnDeclaration.
    def visitColumnDeclaration(self, ctx:PartiQLParser.ColumnDeclarationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#columnConstraint.
    def visitColumnConstraint(self, ctx:PartiQLParser.ColumnConstraintContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ColConstrNotNull.
    def visitColConstrNotNull(self, ctx:PartiQLParser.ColConstrNotNullContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ColConstrNull.
    def visitColConstrNull(self, ctx:PartiQLParser.ColConstrNullContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#DmlBaseWrapper.
    def visitDmlBaseWrapper(self, ctx:PartiQLParser.DmlBaseWrapperContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#DmlDelete.
    def visitDmlDelete(self, ctx:PartiQLParser.DmlDeleteContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#DmlInsertReturning.
    def visitDmlInsertReturning(self, ctx:PartiQLParser.DmlInsertReturningContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#DmlBase.
    def visitDmlBase(self, ctx:PartiQLParser.DmlBaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#dmlBaseCommand.
    def visitDmlBaseCommand(self, ctx:PartiQLParser.DmlBaseCommandContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#pathSimple.
    def visitPathSimple(self, ctx:PartiQLParser.PathSimpleContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#PathSimpleLiteral.
    def visitPathSimpleLiteral(self, ctx:PartiQLParser.PathSimpleLiteralContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#PathSimpleSymbol.
    def visitPathSimpleSymbol(self, ctx:PartiQLParser.PathSimpleSymbolContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#PathSimpleDotSymbol.
    def visitPathSimpleDotSymbol(self, ctx:PartiQLParser.PathSimpleDotSymbolContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#replaceCommand.
    def visitReplaceCommand(self, ctx:PartiQLParser.ReplaceCommandContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#upsertCommand.
    def visitUpsertCommand(self, ctx:PartiQLParser.UpsertCommandContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#removeCommand.
    def visitRemoveCommand(self, ctx:PartiQLParser.RemoveCommandContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#insertCommandReturning.
    def visitInsertCommandReturning(self, ctx:PartiQLParser.InsertCommandReturningContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#insertStatement.
    def visitInsertStatement(self, ctx:PartiQLParser.InsertStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#columnList.
    def visitColumnList(self, ctx:PartiQLParser.ColumnListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#onConflict.
    def visitOnConflict(self, ctx:PartiQLParser.OnConflictContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#insertStatementLegacy.
    def visitInsertStatementLegacy(self, ctx:PartiQLParser.InsertStatementLegacyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#onConflictLegacy.
    def visitOnConflictLegacy(self, ctx:PartiQLParser.OnConflictLegacyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#conflictTarget.
    def visitConflictTarget(self, ctx:PartiQLParser.ConflictTargetContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#constraintName.
    def visitConstraintName(self, ctx:PartiQLParser.ConstraintNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#conflictAction.
    def visitConflictAction(self, ctx:PartiQLParser.ConflictActionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#doReplace.
    def visitDoReplace(self, ctx:PartiQLParser.DoReplaceContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#doUpdate.
    def visitDoUpdate(self, ctx:PartiQLParser.DoUpdateContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#updateClause.
    def visitUpdateClause(self, ctx:PartiQLParser.UpdateClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#setCommand.
    def visitSetCommand(self, ctx:PartiQLParser.SetCommandContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#setAssignment.
    def visitSetAssignment(self, ctx:PartiQLParser.SetAssignmentContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#deleteCommand.
    def visitDeleteCommand(self, ctx:PartiQLParser.DeleteCommandContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#returningClause.
    def visitReturningClause(self, ctx:PartiQLParser.ReturningClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#returningColumn.
    def visitReturningColumn(self, ctx:PartiQLParser.ReturningColumnContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#FromClauseSimpleExplicit.
    def visitFromClauseSimpleExplicit(self, ctx:PartiQLParser.FromClauseSimpleExplicitContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#FromClauseSimpleImplicit.
    def visitFromClauseSimpleImplicit(self, ctx:PartiQLParser.FromClauseSimpleImplicitContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#whereClause.
    def visitWhereClause(self, ctx:PartiQLParser.WhereClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#SelectAll.
    def visitSelectAll(self, ctx:PartiQLParser.SelectAllContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#SelectItems.
    def visitSelectItems(self, ctx:PartiQLParser.SelectItemsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#SelectValue.
    def visitSelectValue(self, ctx:PartiQLParser.SelectValueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#SelectPivot.
    def visitSelectPivot(self, ctx:PartiQLParser.SelectPivotContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#projectionItems.
    def visitProjectionItems(self, ctx:PartiQLParser.ProjectionItemsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#projectionItem.
    def visitProjectionItem(self, ctx:PartiQLParser.ProjectionItemContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#setQuantifierStrategy.
    def visitSetQuantifierStrategy(self, ctx:PartiQLParser.SetQuantifierStrategyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#letClause.
    def visitLetClause(self, ctx:PartiQLParser.LetClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#letBinding.
    def visitLetBinding(self, ctx:PartiQLParser.LetBindingContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#orderByClause.
    def visitOrderByClause(self, ctx:PartiQLParser.OrderByClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#orderSortSpec.
    def visitOrderSortSpec(self, ctx:PartiQLParser.OrderSortSpecContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#groupClause.
    def visitGroupClause(self, ctx:PartiQLParser.GroupClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#groupAlias.
    def visitGroupAlias(self, ctx:PartiQLParser.GroupAliasContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#groupKey.
    def visitGroupKey(self, ctx:PartiQLParser.GroupKeyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#over.
    def visitOver(self, ctx:PartiQLParser.OverContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#windowPartitionList.
    def visitWindowPartitionList(self, ctx:PartiQLParser.WindowPartitionListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#windowSortSpecList.
    def visitWindowSortSpecList(self, ctx:PartiQLParser.WindowSortSpecListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#havingClause.
    def visitHavingClause(self, ctx:PartiQLParser.HavingClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#excludeClause.
    def visitExcludeClause(self, ctx:PartiQLParser.ExcludeClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#excludeExpr.
    def visitExcludeExpr(self, ctx:PartiQLParser.ExcludeExprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExcludeExprTupleAttr.
    def visitExcludeExprTupleAttr(self, ctx:PartiQLParser.ExcludeExprTupleAttrContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExcludeExprCollectionAttr.
    def visitExcludeExprCollectionAttr(self, ctx:PartiQLParser.ExcludeExprCollectionAttrContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExcludeExprCollectionIndex.
    def visitExcludeExprCollectionIndex(self, ctx:PartiQLParser.ExcludeExprCollectionIndexContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExcludeExprCollectionWildcard.
    def visitExcludeExprCollectionWildcard(self, ctx:PartiQLParser.ExcludeExprCollectionWildcardContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExcludeExprTupleWildcard.
    def visitExcludeExprTupleWildcard(self, ctx:PartiQLParser.ExcludeExprTupleWildcardContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#fromClause.
    def visitFromClause(self, ctx:PartiQLParser.FromClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#whereClauseSelect.
    def visitWhereClauseSelect(self, ctx:PartiQLParser.WhereClauseSelectContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#offsetByClause.
    def visitOffsetByClause(self, ctx:PartiQLParser.OffsetByClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#limitClause.
    def visitLimitClause(self, ctx:PartiQLParser.LimitClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#gpmlPattern.
    def visitGpmlPattern(self, ctx:PartiQLParser.GpmlPatternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#gpmlPatternList.
    def visitGpmlPatternList(self, ctx:PartiQLParser.GpmlPatternListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#matchPattern.
    def visitMatchPattern(self, ctx:PartiQLParser.MatchPatternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#graphPart.
    def visitGraphPart(self, ctx:PartiQLParser.GraphPartContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#SelectorBasic.
    def visitSelectorBasic(self, ctx:PartiQLParser.SelectorBasicContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#SelectorAny.
    def visitSelectorAny(self, ctx:PartiQLParser.SelectorAnyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#SelectorShortest.
    def visitSelectorShortest(self, ctx:PartiQLParser.SelectorShortestContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#patternPathVariable.
    def visitPatternPathVariable(self, ctx:PartiQLParser.PatternPathVariableContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#patternRestrictor.
    def visitPatternRestrictor(self, ctx:PartiQLParser.PatternRestrictorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#node.
    def visitNode(self, ctx:PartiQLParser.NodeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#EdgeWithSpec.
    def visitEdgeWithSpec(self, ctx:PartiQLParser.EdgeWithSpecContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#EdgeAbbreviated.
    def visitEdgeAbbreviated(self, ctx:PartiQLParser.EdgeAbbreviatedContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#pattern.
    def visitPattern(self, ctx:PartiQLParser.PatternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#patternQuantifier.
    def visitPatternQuantifier(self, ctx:PartiQLParser.PatternQuantifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#EdgeSpecRight.
    def visitEdgeSpecRight(self, ctx:PartiQLParser.EdgeSpecRightContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#EdgeSpecUndirected.
    def visitEdgeSpecUndirected(self, ctx:PartiQLParser.EdgeSpecUndirectedContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#EdgeSpecLeft.
    def visitEdgeSpecLeft(self, ctx:PartiQLParser.EdgeSpecLeftContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#EdgeSpecUndirectedRight.
    def visitEdgeSpecUndirectedRight(self, ctx:PartiQLParser.EdgeSpecUndirectedRightContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#EdgeSpecUndirectedLeft.
    def visitEdgeSpecUndirectedLeft(self, ctx:PartiQLParser.EdgeSpecUndirectedLeftContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#EdgeSpecBidirectional.
    def visitEdgeSpecBidirectional(self, ctx:PartiQLParser.EdgeSpecBidirectionalContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#EdgeSpecUndirectedBidirectional.
    def visitEdgeSpecUndirectedBidirectional(self, ctx:PartiQLParser.EdgeSpecUndirectedBidirectionalContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#edgeSpec.
    def visitEdgeSpec(self, ctx:PartiQLParser.EdgeSpecContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LabelSpecTerm.
    def visitLabelSpecTerm(self, ctx:PartiQLParser.LabelSpecTermContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LabelSpecOr.
    def visitLabelSpecOr(self, ctx:PartiQLParser.LabelSpecOrContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LabelTermFactor.
    def visitLabelTermFactor(self, ctx:PartiQLParser.LabelTermFactorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LabelTermAnd.
    def visitLabelTermAnd(self, ctx:PartiQLParser.LabelTermAndContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LabelFactorNot.
    def visitLabelFactorNot(self, ctx:PartiQLParser.LabelFactorNotContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LabelFactorPrimary.
    def visitLabelFactorPrimary(self, ctx:PartiQLParser.LabelFactorPrimaryContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LabelPrimaryName.
    def visitLabelPrimaryName(self, ctx:PartiQLParser.LabelPrimaryNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LabelPrimaryWild.
    def visitLabelPrimaryWild(self, ctx:PartiQLParser.LabelPrimaryWildContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LabelPrimaryParen.
    def visitLabelPrimaryParen(self, ctx:PartiQLParser.LabelPrimaryParenContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#edgeAbbrev.
    def visitEdgeAbbrev(self, ctx:PartiQLParser.EdgeAbbrevContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#TableWrapped.
    def visitTableWrapped(self, ctx:PartiQLParser.TableWrappedContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#TableCrossJoin.
    def visitTableCrossJoin(self, ctx:PartiQLParser.TableCrossJoinContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#TableQualifiedJoin.
    def visitTableQualifiedJoin(self, ctx:PartiQLParser.TableQualifiedJoinContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#TableRefBase.
    def visitTableRefBase(self, ctx:PartiQLParser.TableRefBaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#tableNonJoin.
    def visitTableNonJoin(self, ctx:PartiQLParser.TableNonJoinContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#TableBaseRefSymbol.
    def visitTableBaseRefSymbol(self, ctx:PartiQLParser.TableBaseRefSymbolContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#TableBaseRefClauses.
    def visitTableBaseRefClauses(self, ctx:PartiQLParser.TableBaseRefClausesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#TableBaseRefMatch.
    def visitTableBaseRefMatch(self, ctx:PartiQLParser.TableBaseRefMatchContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#tableUnpivot.
    def visitTableUnpivot(self, ctx:PartiQLParser.TableUnpivotContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#JoinRhsBase.
    def visitJoinRhsBase(self, ctx:PartiQLParser.JoinRhsBaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#JoinRhsTableJoined.
    def visitJoinRhsTableJoined(self, ctx:PartiQLParser.JoinRhsTableJoinedContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#joinSpec.
    def visitJoinSpec(self, ctx:PartiQLParser.JoinSpecContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#joinType.
    def visitJoinType(self, ctx:PartiQLParser.JoinTypeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#expr.
    def visitExpr(self, ctx:PartiQLParser.ExprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#Intersect.
    def visitIntersect(self, ctx:PartiQLParser.IntersectContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#QueryBase.
    def visitQueryBase(self, ctx:PartiQLParser.QueryBaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#Except.
    def visitExcept(self, ctx:PartiQLParser.ExceptContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#Union.
    def visitUnion(self, ctx:PartiQLParser.UnionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#SfwQuery.
    def visitSfwQuery(self, ctx:PartiQLParser.SfwQueryContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#SfwBase.
    def visitSfwBase(self, ctx:PartiQLParser.SfwBaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#Or.
    def visitOr(self, ctx:PartiQLParser.OrContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExprOrBase.
    def visitExprOrBase(self, ctx:PartiQLParser.ExprOrBaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExprAndBase.
    def visitExprAndBase(self, ctx:PartiQLParser.ExprAndBaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#And.
    def visitAnd(self, ctx:PartiQLParser.AndContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#Not.
    def visitNot(self, ctx:PartiQLParser.NotContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExprNotBase.
    def visitExprNotBase(self, ctx:PartiQLParser.ExprNotBaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#PredicateIn.
    def visitPredicateIn(self, ctx:PartiQLParser.PredicateInContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#PredicateBetween.
    def visitPredicateBetween(self, ctx:PartiQLParser.PredicateBetweenContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#PredicateBase.
    def visitPredicateBase(self, ctx:PartiQLParser.PredicateBaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#PredicateComparison.
    def visitPredicateComparison(self, ctx:PartiQLParser.PredicateComparisonContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#PredicateIs.
    def visitPredicateIs(self, ctx:PartiQLParser.PredicateIsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#PredicateLike.
    def visitPredicateLike(self, ctx:PartiQLParser.PredicateLikeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#mathOp00.
    def visitMathOp00(self, ctx:PartiQLParser.MathOp00Context):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#mathOp01.
    def visitMathOp01(self, ctx:PartiQLParser.MathOp01Context):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#mathOp02.
    def visitMathOp02(self, ctx:PartiQLParser.MathOp02Context):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#valueExpr.
    def visitValueExpr(self, ctx:PartiQLParser.ValueExprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExprPrimaryPath.
    def visitExprPrimaryPath(self, ctx:PartiQLParser.ExprPrimaryPathContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExprPrimaryBase.
    def visitExprPrimaryBase(self, ctx:PartiQLParser.ExprPrimaryBaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExprTermWrappedQuery.
    def visitExprTermWrappedQuery(self, ctx:PartiQLParser.ExprTermWrappedQueryContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExprTermCurrentUser.
    def visitExprTermCurrentUser(self, ctx:PartiQLParser.ExprTermCurrentUserContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExprTermCurrentDate.
    def visitExprTermCurrentDate(self, ctx:PartiQLParser.ExprTermCurrentDateContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#ExprTermBase.
    def visitExprTermBase(self, ctx:PartiQLParser.ExprTermBaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#nullIf.
    def visitNullIf(self, ctx:PartiQLParser.NullIfContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#coalesce.
    def visitCoalesce(self, ctx:PartiQLParser.CoalesceContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#caseExpr.
    def visitCaseExpr(self, ctx:PartiQLParser.CaseExprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#values.
    def visitValues(self, ctx:PartiQLParser.ValuesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#valueRow.
    def visitValueRow(self, ctx:PartiQLParser.ValueRowContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#valueList.
    def visitValueList(self, ctx:PartiQLParser.ValueListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#sequenceConstructor.
    def visitSequenceConstructor(self, ctx:PartiQLParser.SequenceConstructorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#substring.
    def visitSubstring(self, ctx:PartiQLParser.SubstringContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#position.
    def visitPosition(self, ctx:PartiQLParser.PositionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#overlay.
    def visitOverlay(self, ctx:PartiQLParser.OverlayContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#CountAll.
    def visitCountAll(self, ctx:PartiQLParser.CountAllContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#AggregateBase.
    def visitAggregateBase(self, ctx:PartiQLParser.AggregateBaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LagLeadFunction.
    def visitLagLeadFunction(self, ctx:PartiQLParser.LagLeadFunctionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#cast.
    def visitCast(self, ctx:PartiQLParser.CastContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#canLosslessCast.
    def visitCanLosslessCast(self, ctx:PartiQLParser.CanLosslessCastContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#canCast.
    def visitCanCast(self, ctx:PartiQLParser.CanCastContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#extract.
    def visitExtract(self, ctx:PartiQLParser.ExtractContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#trimFunction.
    def visitTrimFunction(self, ctx:PartiQLParser.TrimFunctionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#dateFunction.
    def visitDateFunction(self, ctx:PartiQLParser.DateFunctionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#functionCall.
    def visitFunctionCall(self, ctx:PartiQLParser.FunctionCallContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#FunctionNameReserved.
    def visitFunctionNameReserved(self, ctx:PartiQLParser.FunctionNameReservedContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#FunctionNameSymbol.
    def visitFunctionNameSymbol(self, ctx:PartiQLParser.FunctionNameSymbolContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#PathStepIndexExpr.
    def visitPathStepIndexExpr(self, ctx:PartiQLParser.PathStepIndexExprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#PathStepIndexAll.
    def visitPathStepIndexAll(self, ctx:PartiQLParser.PathStepIndexAllContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#PathStepDotExpr.
    def visitPathStepDotExpr(self, ctx:PartiQLParser.PathStepDotExprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#PathStepDotAll.
    def visitPathStepDotAll(self, ctx:PartiQLParser.PathStepDotAllContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#exprGraphMatchMany.
    def visitExprGraphMatchMany(self, ctx:PartiQLParser.ExprGraphMatchManyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#exprGraphMatchOne.
    def visitExprGraphMatchOne(self, ctx:PartiQLParser.ExprGraphMatchOneContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#parameter.
    def visitParameter(self, ctx:PartiQLParser.ParameterContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#VariableIdentifier.
    def visitVariableIdentifier(self, ctx:PartiQLParser.VariableIdentifierContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#VariableKeyword.
    def visitVariableKeyword(self, ctx:PartiQLParser.VariableKeywordContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#nonReservedKeywords.
    def visitNonReservedKeywords(self, ctx:PartiQLParser.NonReservedKeywordsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#collection.
    def visitCollection(self, ctx:PartiQLParser.CollectionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#array.
    def visitArray(self, ctx:PartiQLParser.ArrayContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#bag.
    def visitBag(self, ctx:PartiQLParser.BagContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#tuple.
    def visitTuple(self, ctx:PartiQLParser.TupleContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#pair.
    def visitPair(self, ctx:PartiQLParser.PairContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LiteralNull.
    def visitLiteralNull(self, ctx:PartiQLParser.LiteralNullContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LiteralMissing.
    def visitLiteralMissing(self, ctx:PartiQLParser.LiteralMissingContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LiteralTrue.
    def visitLiteralTrue(self, ctx:PartiQLParser.LiteralTrueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LiteralFalse.
    def visitLiteralFalse(self, ctx:PartiQLParser.LiteralFalseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LiteralString.
    def visitLiteralString(self, ctx:PartiQLParser.LiteralStringContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LiteralInteger.
    def visitLiteralInteger(self, ctx:PartiQLParser.LiteralIntegerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LiteralDecimal.
    def visitLiteralDecimal(self, ctx:PartiQLParser.LiteralDecimalContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LiteralIon.
    def visitLiteralIon(self, ctx:PartiQLParser.LiteralIonContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LiteralDate.
    def visitLiteralDate(self, ctx:PartiQLParser.LiteralDateContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LiteralTime.
    def visitLiteralTime(self, ctx:PartiQLParser.LiteralTimeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#LiteralTimestamp.
    def visitLiteralTimestamp(self, ctx:PartiQLParser.LiteralTimestampContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#TypeAtomic.
    def visitTypeAtomic(self, ctx:PartiQLParser.TypeAtomicContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#TypeArgSingle.
    def visitTypeArgSingle(self, ctx:PartiQLParser.TypeArgSingleContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#TypeVarChar.
    def visitTypeVarChar(self, ctx:PartiQLParser.TypeVarCharContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#TypeArgDouble.
    def visitTypeArgDouble(self, ctx:PartiQLParser.TypeArgDoubleContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#TypeTimeZone.
    def visitTypeTimeZone(self, ctx:PartiQLParser.TypeTimeZoneContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by PartiQLParser#TypeCustom.
    def visitTypeCustom(self, ctx:PartiQLParser.TypeCustomContext):
        return self.visitChildren(ctx)



del PartiQLParser


# -*- coding: utf-8 -*-
import logging
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional, Union

from .builder import ExecutionPlan

_logger = logging.getLogger(__name__)


@dataclass
class QueryExecutionPlan(ExecutionPlan):
    """Execution plan for MongoDB SELECT queries (query-only)."""

    filter_stage: Dict[str, Any] = field(default_factory=dict)
    projection_stage: Dict[str, Any] = field(default_factory=dict)
    column_aliases: Dict[str, str] = field(default_factory=dict)  # Maps field_name -> alias
    sort_stage: List[Dict[str, int]] = field(default_factory=list)
    limit_stage: Optional[int] = None
    skip_stage: Optional[int] = None
    # Aggregate pipeline support
    aggregate_pipeline: Optional[str] = None  # JSON string representation of pipeline
    aggregate_options: Optional[str] = None  # JSON string representation of options
    is_aggregate_query: bool = False  # Flag indicating this is an aggregate() call

    def to_dict(self) -> Dict[str, Any]:
        """Convert query plan to dictionary representation"""
        result = {
            "collection": self.collection,
            "filter": self.filter_stage,
            "projection": self.projection_stage,
            "sort": self.sort_stage,
            "limit": self.limit_stage,
            "skip": self.skip_stage,
        }

        # Add aggregate-specific fields if present
        if self.is_aggregate_query:
            result["is_aggregate_query"] = True
            result["aggregate_pipeline"] = self.aggregate_pipeline
            result["aggregate_options"] = self.aggregate_options

        return result

    def validate(self) -> bool:
        """Validate the query plan"""
        # For aggregate queries, collection is optional (unqualified aggregate syntax)
        # For regular queries, collection is required
        if self.is_aggregate_query:
            errors = []
        else:
            errors = self.validate_base()

        if self.limit_stage is not None and (not isinstance(self.limit_stage, int) or self.limit_stage < 0):
            errors.append("Limit must be a non-negative integer")

        if self.skip_stage is not None and (not isinstance(self.skip_stage, int) or self.skip_stage < 0):
            errors.append("Skip must be a non-negative integer")

        if errors:
            _logger.error(f"Query validation errors: {errors}")
            return False

        return True

    def copy(self) -> "QueryExecutionPlan":
        """Create a copy of this execution plan"""
        return QueryExecutionPlan(
            collection=self.collection,
            filter_stage=self.filter_stage.copy(),
            projection_stage=self.projection_stage.copy(),
            column_aliases=self.column_aliases.copy(),
            sort_stage=self.sort_stage.copy(),
            limit_stage=self.limit_stage,
            skip_stage=self.skip_stage,
            aggregate_pipeline=self.aggregate_pipeline,
            aggregate_options=self.aggregate_options,
            is_aggregate_query=self.is_aggregate_query,
        )


class MongoQueryBuilder:
    """Fluent builder for MongoDB queries with validation and readability"""

    def __init__(self):
        self._execution_plan = QueryExecutionPlan()
        self._validation_errors = []

    def collection(self, name: str) -> "MongoQueryBuilder":
        """Set the target collection"""
        if not name or not name.strip():
            self._add_error("Collection name cannot be empty")
            return self

        self._execution_plan.collection = name.strip()
        _logger.debug(f"Set collection to: {name}")
        return self

    def filter(self, conditions: Dict[str, Any]) -> "MongoQueryBuilder":
        """Add filter conditions"""
        if not isinstance(conditions, dict):
            self._add_error("Filter conditions must be a dictionary")
            return self

        self._execution_plan.filter_stage.update(conditions)
        _logger.debug(f"Added filter conditions: {conditions}")
        return self

    def project(self, fields: Union[Dict[str, int], List[str]]) -> "MongoQueryBuilder":
        """Set projection fields"""
        if isinstance(fields, list):
            # Convert list to projection dict
            projection = {field: 1 for field in fields}
        elif isinstance(fields, dict):
            projection = fields
        else:
            self._add_error("Projection must be a list of field names or a dictionary")
            return self

        self._execution_plan.projection_stage = projection
        _logger.debug(f"Set projection: {projection}")
        return self

    def sort(self, specs: List[Dict[str, int]]) -> "MongoQueryBuilder":
        """Add sort criteria.

        Only accepts a list of single-key dicts in the form:
            [{"field": 1}, {"other": -1}]

        This matches the output produced by the SQL parser (`sort_fields`).
        """
        if not isinstance(specs, list):
            self._add_error("Sort specifications must be a list of single-key dicts")
            return self

        for spec in specs:
            if not isinstance(spec, dict) or len(spec) != 1:
                self._add_error("Each sort specification must be a single-key dict, e.g. {'name': 1}")
                continue

            field, direction = next(iter(spec.items()))

            if not isinstance(field, str) or not field:
                self._add_error("Sort field must be a non-empty string")
                continue

            if direction not in [-1, 1]:
                self._add_error(f"Sort direction for field '{field}' must be 1 or -1")
                continue

            self._execution_plan.sort_stage.append({field: direction})
            _logger.debug(f"Added sort: {field} -> {direction}")

        return self

    def limit(self, count: int) -> "MongoQueryBuilder":
        """Set limit for results"""
        if not isinstance(count, int) or count < 0:
            self._add_error("Limit must be a non-negative integer")
            return self

        self._execution_plan.limit_stage = count
        _logger.debug(f"Set limit to: {count}")
        return self

    def skip(self, count: int) -> "MongoQueryBuilder":
        """Set skip count for pagination"""
        if not isinstance(count, int) or count < 0:
            self._add_error("Skip must be a non-negative integer")
            return self

        self._execution_plan.skip_stage = count
        _logger.debug(f"Set skip to: {count}")
        return self

    def column_aliases(self, aliases: Dict[str, str]) -> "MongoQueryBuilder":
        """Set column aliases mapping (field_name -> alias)"""
        if not isinstance(aliases, dict):
            self._add_error("Column aliases must be a dictionary")
            return self

        self._execution_plan.column_aliases = aliases
        _logger.debug(f"Set column aliases to: {aliases}")
        return self

    def where(self, field: str, operator: str, value: Any) -> "MongoQueryBuilder":
        """Add a where condition in a readable format"""
        condition = self._build_condition(field, operator, value)
        if condition:
            return self.filter(condition)
        return self

    def where_in(self, field: str, values: List[Any]) -> "MongoQueryBuilder":
        """Add a WHERE field IN (values) condition"""
        return self.filter({field: {"$in": values}})

    def where_between(self, field: str, min_val: Any, max_val: Any) -> "MongoQueryBuilder":
        """Add a WHERE field BETWEEN min AND max condition"""
        return self.filter({field: {"$gte": min_val, "$lte": max_val}})

    def where_like(self, field: str, pattern: str) -> "MongoQueryBuilder":
        """Add a WHERE field LIKE pattern condition"""
        # Convert SQL LIKE pattern to MongoDB regex
        regex_pattern = pattern.replace("%", ".*").replace("_", ".")
        return self.filter({field: {"$regex": regex_pattern, "$options": "i"}})

    def _build_condition(self, field: str, operator: str, value: Any) -> Optional[Dict[str, Any]]:
        """Build a MongoDB condition from field, operator, and value"""
        operator_map = {
            "=": "$eq",
            "!=": "$ne",
            "<": "$lt",
            "<=": "$lte",
            ">": "$gt",
            ">=": "$gte",
            "eq": "$eq",
            "ne": "$ne",
            "lt": "$lt",
            "lte": "$lte",
            "gt": "$gt",
            "gte": "$gte",
        }

        mongo_op = operator_map.get(operator.lower())
        if not mongo_op:
            self._add_error(f"Unsupported operator: {operator}")
            return None

        return {field: {mongo_op: value}}

    def _add_error(self, message: str) -> None:
        """Add validation error"""
        self._validation_errors.append(message)
        _logger.error(f"Query builder error: {message}")

    def validate(self) -> bool:
        """Validate the current query plan"""
        self._validation_errors.clear()

        # For aggregate queries, collection is optional (unqualified aggregate syntax)
        # For regular queries, collection is required
        if not self._execution_plan.is_aggregate_query and not self._execution_plan.collection:
            self._add_error("Collection name is required")

        # Add more validation rules as needed
        return len(self._validation_errors) == 0

    def get_errors(self) -> List[str]:
        """Get validation errors"""
        return self._validation_errors.copy()

    def build(self) -> QueryExecutionPlan:
        """Build and return the execution plan"""
        if not self.validate():
            error_summary = "; ".join(self._validation_errors)
            raise ValueError(f"Query validation failed: {error_summary}")

        return self._execution_plan

    def reset(self) -> "MongoQueryBuilder":
        """Reset the builder to start a new query"""
        self._execution_plan = QueryExecutionPlan()
        self._validation_errors.clear()
        return self

    def __str__(self) -> str:
        """String representation for debugging"""
        return (
            f"MongoQueryBuilder(collection={self._execution_plan.collection}, "
            f"filter={self._execution_plan.filter_stage}, "
            f"projection={self._execution_plan.projection_stage})"
        )



# -*- coding: utf-8 -*-
import logging
import re
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional, Tuple

from .handler import BaseHandler, ContextUtilsMixin
from .partiql.PartiQLParser import PartiQLParser

_logger = logging.getLogger(__name__)


@dataclass
class QueryParseResult:
    """Result container for query (SELECT) expression parsing and visitor state management"""

    # Core parsing fields
    filter_conditions: Dict[str, Any] = field(default_factory=dict)  # Unified filter field for all MongoDB conditions
    has_errors: bool = False
    error_message: Optional[str] = None

    # Visitor parsing state fields
    collection: Optional[str] = None
    projection: Dict[str, Any] = field(default_factory=dict)
    column_aliases: Dict[str, str] = field(default_factory=dict)  # Maps field_name -> alias
    sort_fields: List[Dict[str, int]] = field(default_factory=list)
    limit_value: Optional[int] = None
    offset_value: Optional[int] = None

    # Aggregate pipeline support
    is_aggregate_query: bool = False  # Flag indicating this is an aggregate() call
    aggregate_pipeline: Optional[str] = None  # JSON string representation of pipeline
    aggregate_options: Optional[str] = None  # JSON string representation of options

    # Subquery info (for wrapped subqueries, e.g., Superset outering)
    subquery_plan: Optional[Any] = None
    subquery_alias: Optional[str] = None

    # Factory methods for different use cases
    @classmethod
    def for_visitor(cls) -> "QueryParseResult":
        """Create QueryParseResult for visitor parsing"""
        return cls()

    def merge_expression(self, other: "QueryParseResult") -> "QueryParseResult":
        """Merge expression results from another QueryParseResult"""
        if other.has_errors:
            self.has_errors = True
            self.error_message = other.error_message

        # Merge filter conditions intelligently
        if other.filter_conditions:
            if not self.filter_conditions:
                self.filter_conditions = other.filter_conditions
            else:
                # If both have filters, combine them with $and
                self.filter_conditions = {"$and": [self.filter_conditions, other.filter_conditions]}

        return self

    # Backward compatibility properties
    @property
    def mongo_filter(self) -> Dict[str, Any]:
        """Backward compatibility property for mongo_filter"""
        return self.filter_conditions

    @mongo_filter.setter
    def mongo_filter(self, value: Dict[str, Any]):
        """Backward compatibility setter for mongo_filter"""
        self.filter_conditions = value


class EnhancedWhereHandler(ContextUtilsMixin):
    """Enhanced WHERE clause handler using expression handlers"""

    def handle(self, ctx: PartiQLParser.WhereClauseSelectContext) -> Dict[str, Any]:
        """Handle WHERE clause with proper expression parsing"""
        if not hasattr(ctx, "exprSelect") or not ctx.exprSelect():
            _logger.debug("No expression found in WHERE clause")
            return {}

        expression_ctx = ctx.exprSelect()
        # Local import to avoid circular dependency between query_handler and handler
        from .handler import HandlerFactory

        handler = HandlerFactory.get_expression_handler(expression_ctx)

        if handler:
            _logger.debug(
                f"Using {type(handler).__name__} for WHERE clause",
                extra={"context_text": self.get_context_text(expression_ctx)[:100]},
            )
            result = handler.handle_expression(expression_ctx)
            if result.has_errors:
                _logger.warning(
                    "Expression parsing error, falling back to text search",
                    extra={"error": result.error_message},
                )
                # Fallback to text-based filter
                return {"$text": {"$search": self.get_context_text(expression_ctx)}}
            return result.filter_conditions
        else:
            # Fallback to simple text-based search
            _logger.debug(
                "No suitable expression handler found, using text search",
                extra={"context_text": self.get_context_text(expression_ctx)[:100]},
            )
            return {"$text": {"$search": self.get_context_text(expression_ctx)}}


class SelectHandler(BaseHandler, ContextUtilsMixin):
    """Handles SELECT statement parsing"""

    def can_handle(self, ctx: Any) -> bool:
        """Check if this is a select context"""
        return hasattr(ctx, "projectionItems")

    def handle_visitor(self, ctx: PartiQLParser.SelectItemsContext, parse_result: "QueryParseResult") -> Any:
        projection = {}
        column_aliases = {}

        if hasattr(ctx, "projectionItems") and ctx.projectionItems():
            for item in ctx.projectionItems().projectionItem():
                field_name, alias = self._extract_field_and_alias(item)
                # Use MongoDB standard projection format: {field: 1} to include field
                projection[field_name] = 1
                # Store alias if present
                if alias:
                    column_aliases[field_name] = alias

        parse_result.projection = projection
        parse_result.column_aliases = column_aliases
        return projection

    def _extract_field_and_alias(self, item) -> Tuple[str, Optional[str]]:
        """Extract field name and alias from projection item context with nested field support"""
        if not hasattr(item, "children") or not item.children:
            return str(item), None

        # According to grammar: projectionItem : expr ( AS? symbolPrimitive )? ;
        # children[0] is always the expression
        # If there's an alias, children[1] might be AS and children[2] symbolPrimitive
        # OR children[1] might be just symbolPrimitive (without AS)

        field_name = item.children[0].getText()
        # Normalize bracket notation (jmspath) to Mongo dot notation
        field_name = self.normalize_field_path(field_name)

        alias = None

        if len(item.children) >= 2:
            # Check if we have an alias
            if len(item.children) == 3:
                # Pattern: expr AS symbolPrimitive
                if hasattr(item.children[1], "getText") and item.children[1].getText().upper() == "AS":
                    alias = item.children[2].getText()
            elif len(item.children) == 2:
                # Pattern: expr symbolPrimitive (without AS)
                alias = item.children[1].getText()

        return field_name, alias


class FromHandler(BaseHandler):
    """Handles FROM clause parsing with support for regular collections and aggregate() function calls"""

    def can_handle(self, ctx: Any) -> bool:
        """Check if this is a from context"""
        return hasattr(ctx, "tableReference")

    def _parse_function_call(self, ctx: Any) -> Optional[Dict[str, Any]]:
        """
        Detect and parse aggregate() function calls in FROM clause.

        Supports:
        - collection.aggregate('pipeline_json', 'options_json')
        - aggregate('pipeline_json', 'options_json')

        Returns dict with:
        - function_name: 'aggregate'
        - collection: collection name (or None if unqualified)
        - pipeline: JSON string for pipeline
        - options: JSON string for options
        """
        try:
            # Get the tableReference from FROM clause
            if not hasattr(ctx, "tableReference"):
                return None

            table_ref = ctx.tableReference()
            if not table_ref:
                return None

            # Get the text to analyze
            text = table_ref.getText() if hasattr(table_ref, "getText") else str(table_ref)

            # Pattern: [qualifier.]functionName(arg1, arg2)
            # We need to match: (optional_collection.)aggregate('...', '...')
            pattern = r"^(?:(\w+)\.)?aggregate\s*\(\s*'([^']*)'\s*,\s*'([^']*)'\s*\)$"
            match = re.match(pattern, text, re.IGNORECASE | re.DOTALL)

            if not match:
                return None

            collection = match.group(1)  # Can be None for unqualified aggregate()
            pipeline = match.group(2)
            options = match.group(3)

            _logger.debug(
                f"Detected aggregate call: collection={collection}, pipeline={pipeline[:50]}..., options={options}"
            )

            return {
                "function_name": "aggregate",
                "collection": collection,
                "pipeline": pipeline,
                "options": options,
            }
        except Exception as e:
            _logger.debug(f"Error parsing function call: {e}")
            return None

    def handle_visitor(self, ctx: PartiQLParser.FromClauseContext, parse_result: "QueryParseResult") -> Any:
        """Handle FROM clause - detect aggregate calls or regular collections"""
        if hasattr(ctx, "tableReference") and ctx.tableReference():
            # Try to detect aggregate function call
            func_info = self._parse_function_call(ctx)

            if func_info and func_info["function_name"] == "aggregate":
                # Mark as aggregate query
                if hasattr(parse_result, "is_aggregate_query"):
                    parse_result.is_aggregate_query = True
                if hasattr(parse_result, "aggregate_pipeline"):
                    parse_result.aggregate_pipeline = func_info["pipeline"]
                if hasattr(parse_result, "aggregate_options"):
                    parse_result.aggregate_options = func_info["options"]

                # Set collection name if qualified, otherwise it's collection-agnostic
                if func_info["collection"]:
                    parse_result.collection = func_info["collection"]

                _logger.info(f"Parsed aggregate call: collection={func_info['collection']}")
                return func_info

            # Regular collection reference
            table_text = ctx.tableReference().getText()
            collection_name = table_text
            parse_result.collection = collection_name
            _logger.debug(f"Parsed regular collection: {collection_name}")
            return collection_name

        return None


class WhereHandler(BaseHandler):
    """Handles WHERE clause parsing"""

    def __init__(self):
        self._expression_handler = EnhancedWhereHandler()

    def can_handle(self, ctx: Any) -> bool:
        """Check if this is a where context"""
        return hasattr(ctx, "exprSelect")

    def handle_visitor(self, ctx: PartiQLParser.WhereClauseSelectContext, parse_result: "QueryParseResult") -> Any:
        if hasattr(ctx, "exprSelect") and ctx.exprSelect():
            try:
                # Use enhanced expression handler for better parsing
                filter_conditions = self._expression_handler.handle(ctx)
                parse_result.filter_conditions = filter_conditions
                return filter_conditions
            except Exception as e:
                _logger.warning(f"Failed to parse WHERE expression, falling back to text search: {e}")
                # Fallback to simple text search
                filter_text = ctx.exprSelect().getText()
                fallback_filter = {"$text": {"$search": filter_text}}
                parse_result.filter_conditions = fallback_filter
                return fallback_filter
        return {}
